package com.enjoy.shiyi.servlet;

import com.enjoy.shiyi.annaotaion.*;
import com.enjoy.shiyi.controller.ShiyiController;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName : DispatcherServlet
 * @Description :
 * @Author : YHY
 * @Date: 2023-10-28 20:09
 */
public class DispatcherServlet extends HttpServlet {

    List<String> className = new ArrayList<String>();

    Map<String, Object> beans = new HashMap<String,Object>();
    Map<String, Object> handlerMap = new HashMap<String,Object>();
    private static final long serialVersionUID = 1L;

    @Override
    public void init(ServletConfig config){
        //把所有的bean扫描-----> 扫描所有的class文件
        scanPackage("com.enjoy"); //写到properties

        doInstance();//根据全类名创建bean

        doIoc(); //根据bean进行依赖注入

        buildURLMapping(); //shiyi/query-----> method 建立映射关系

    }

    private void scanPackage(String basePackage){
        URL url = this.getClass().getClassLoader().getResource("/"+basePackage.replaceAll("\\.","/"));
        String fileStr = url.getFile();

        File file = new File(fileStr);

        String[] filesStr = file.list();

        for(String path: filesStr){
            File filePath = new File(fileStr + path);

            if (filePath.isDirectory()){
                scanPackage(basePackage+"."+path);
            }else {
                //加入LIST
                className.add(basePackage+"."+filePath.getName()); //路径+class类名
            }
        }
    }

    //根据扫描出来的LIST全类名，进行实例化
    private void doInstance(){
        if (className.size() <= 0){
            System.out.println("包扫描失败....");
            return;
        }


        //list的所有class类，对这些类进行实例化
        for (String className:className){
            String cn = className.replace(".class","");
            try {
                Class<?> clazz = Class.forName(cn);
                if (clazz.isAnnotationPresent(EnjoyController.class)){
                    Object instace = clazz.newInstance();  //创建控制类

                    EnjoyRequestMapping requestMapping = clazz.getAnnotation(EnjoyRequestMapping.class);
                    String rmvalue = requestMapping.value();
                    beans.put(rmvalue, instace);
                }else if (clazz.isAnnotationPresent(EnjoyService.class)){
                    EnjoyService service = clazz.getAnnotation(EnjoyService.class);
                    Object instace = clazz.newInstance();
                    beans.put(service.value(), instace);
                }else {
                    continue;
                }

            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            } catch (InstantiationException e) {
                throw new RuntimeException(e);
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        }
    }


    //把service注入到controller
    public void doIoc(){
        if (beans.entrySet().size() <= 0){
            System.out.println("没有一个实例化类");
        }

        for (Map.Entry<String,Object> entry:beans.entrySet()){

            Object instace = entry.getValue();
            Class<?> clazz = instace.getClass();

            if (clazz.isAnnotationPresent(EnjoyController.class)){
                Field[] fileds = clazz.getDeclaredFields();
                for (Field filed : fileds) {
                    if (filed.isAnnotationPresent(EnjoyAutowired.class)){
                        EnjoyAutowired auto = filed.getAnnotation(EnjoyAutowired.class);
                        String key = auto.value();
                        //打开权限 才能设值
                        filed.setAccessible(true);
                        try {
                            //对当前成员变量进行射设值
                            filed.set(instace, beans.get(key));
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        }
                    }else {
                        continue;
                    }
                }
            }else {
                continue;
            }


        }
    }

    private void buildURLMapping(){
        if (beans.entrySet().size() <=0 ){
            System.out.println("没有类的实例化------");
            return;
        }

        for (Map.Entry<String, Object> entry: beans.entrySet()){
            Object instace = entry.getValue();

            Class<?> clazz = instace.getClass();
            if (clazz.isAnnotationPresent(EnjoyController.class)){
                EnjoyRequestMapping requestMapping = clazz.getAnnotation(EnjoyRequestMapping.class);
                String classPath = requestMapping.value();

                Method[] methods = clazz.getMethods();
                for (Method method : methods) {
                    if (method.isAnnotationPresent(EnjoyRequestMapping.class)){
                        EnjoyRequestMapping methodMapping = method.getAnnotation(EnjoyRequestMapping.class);
                        String methodPath = methodMapping.value();//方法的路径

                        handlerMap.put(classPath + methodPath, method);
                    }else {
                        continue;
                    }
                }
            }else {
                continue;
            }
        }
    }


    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        this.doPost(req,resp);
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        //获取请求路径 /shiyi-mvc/shiyi/query
        String url = req.getRequestURI();
        String context = req.getContextPath(); //shiyi-mvc
        String path = url.replace(context, ""); //shiyi/query

        Method method = (Method) handlerMap.get(path);

        //根据key=/shiyi到map去拿
        ShiyiController instance = (ShiyiController) beans.get("/" + path.split("/")[1]);

        Object arg[] = hand(req,resp,method);
        try {
            method.invoke(instance,arg);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        }
    }

    //这里不用策略模式，VIP才有策略模式
    private static Object[] hand(HttpServletRequest request, HttpServletResponse response,Method method){
        //拿到当前待执行方法有哪些参数
        Class<?>[] paramClazzs = method.getParameterTypes();
        //根据参数的个数，new 一个参数的数组，讲方法里的所有参数赋值到args来
        Object[] args = new Object[paramClazzs.length];

        int args_i = 0;
        int index = 0;
        for (Class<?> paramClazz : paramClazzs) {
            if (ServletRequest.class.isAssignableFrom(paramClazz)){
                args[args_i++] = request;
            }
            if (ServletResponse.class.isAssignableFrom(paramClazz)){
                args[args_i++] = response;
            }
            //从0-3判断有没有RequestParam注解，很明显paramClazz为0和1时，不是
            //当为2时为@RequestParam，需要解析
            //[@com.enjoy.shiyi.annaotaion.EnjoyRequestMapping(value=name)]
            Annotation[] paramAns = method.getParameterAnnotations()[index];
            if (paramAns.length > 0){
                for (Annotation paramAn : paramAns) {
                    if (EnjoyRequestParm.class.isAssignableFrom(paramAn.getClass())){
                        EnjoyRequestParm rp = (EnjoyRequestParm) paramAn;
                        //找到注解里的name和age
                        args[args_i++] = request.getParameter(rp.value());
                    }
                }
            }
            index++;
        }
        return args;
    }

}
